@@ -15,7 +15,7 @@ __author__ = "Ali Onur Uyar"
 __copyright__ = "Copyright 2011, Ali Onur Uyar"
 __credits__ = ["Samuel Stauffer"]
 __license__ = "GPL"
-__version__ = "0.9.7"
+__version__ = "0.9.9"
 __maintainer__ = "Ali Onur Uyar"
 __email__ = "aouyar at gmail.com"
 __status__ = "Development"
@@ -25,6 +25,7 @@ maxLabelLenGraphSimple = 40
 maxLabelLenGraphDual = 14
 
 
+
 class MuninAttrFilter:
     """Class for implementing Attribute Filters for Munin Graphs.
     
@@ -72,6 +73,7 @@ class MuninAttrFilter:
         Returns True if the attribute is enabled, false otherwise.
         
         @param attr: Name of attribute.
+        @return:     Boolean
         
         """
         return self._attrs.get(attr, self._default)
@@ -104,7 +106,8 @@ class MuninPlugin:
         """
         self._graphDict = {}
         self._graphNames = []
-        self._subGraphDict = {}
+        self._subgraphDict = {}
+        self._subgraphNames = {}
         self._filters = {}
         self._flags = {}
         self._argv = argv
@@ -122,8 +125,8 @@ class MuninPlugin:
         self.envRegisterFilter('graphs', '^[\w\-]+$')
         self._nestedGraphs = self.envCheckFlag('nested_graphs', True)
                 
-    def _parseEnv(self,  env=None):
-        """Utility method that parses through environment variables.
+    def _parseEnv(self, env=None):
+        """Private method for parsing through environment variables.
         
         Parses for environment variables common to all Munin Plugins:
             - MUNIN_STATEFILE
@@ -131,7 +134,8 @@ class MuninPlugin:
             - nested_graphs
         
         @param env: Dictionary of environment variables.
-                    (Only used for testing. initialized automatically by constructor.
+                    (Only used for testing. initialized automatically by 
+                    constructor.
         
         """
         if not env:
@@ -142,25 +146,79 @@ class MuninPlugin:
             self._stateFile = '/tmp/munin-state-%s' % self.plugin_name
         if env.has_key('MUNIN_CAP_DIRTY_CONFIG'):
             self._dirtyConfig = True
-       
+            
+    def _getGraph(self, graph_name, fail_noexist=False):
+        """Private method for returning graph object with name graph_name. 
+        
+        @param graph_name:   Graph Name
+        @param fail_noexist: If true throw exception if there is no graph with
+                             name graph_name.
+        @return:             Graph Object or None
+        """
+        graph = self._graphDict.get(graph_name)
+        if fail_noexist and graph is None:
+            raise AttributeError("Invalid graph name %s", graph_name)
+        else:
+            return graph
+            
+    def _getSubGraph(self, parent_name, graph_name, fail_noexist=False):
+        """Private method for returning subgraph object with name graph_name 
+        and parent graph with name parent_name. 
+        
+        @param  parent_name: Root Graph Name
+        @param  graph_name:  Subgraph Name
+        @param fail_noexist: If true throw exception if there is no subgraph 
+                             with name graph_name.
+        @return:             Graph Object or None
+        """
+        if not self.isMultigraph:
+            raise AttributeError("Simple Munin Plugins cannot have subgraphs.")
+        if self._graphDict.has_key(parent_name) is not None:
+            subgraphs = self._subgraphDict.get(parent_name)
+            if subgraphs is not None:
+                subgraph = subgraphs.get(graph_name)
+                if fail_noexist and subgraph is None:
+                    raise AttributeError("Invalid subgraph name %s"
+                                         "for graph %s.", 
+                                         (graph_name, parent_name))
+                else:
+                    return subgraph
+            else:
+                raise AttributeError("Parent graph %s has no subgraphs."
+                                     % (parent_name,))
+        else:
+            raise AttributeError("Invalid parent graph name %s "
+                                 "for subgraph %s."
+                                 % (parent_name, graph_name))
+            
     def envHasKey(self, name):
         """Return True if environment variable with name exists.  
         
         @param name: Name of environtment variable.
+        @return:     True if environment variable is defined.
         
         """
         return self._env.has_key(name)
     
-    def envGet(self, name, default=None):
+    def envGet(self, name, default=None, conv=None):
         """Return value for environment variable or None.  
         
         @param name:    Name of environment variable.
         @param default: Default value if variable is undefined.
+        @param conv:    Function for converting value to desired type.
+        @return:        Value of environment variable.
         
         """
-        return self._env.get(name, default)
+        if self._env.has_key(name):
+            if conv is not None:
+                return conv(self._env.get(name))
+            else:
+                return self._env.get(name)
+        else:
+            return default
+        
     
-    def envGetList(self, name, attr_regex = '^\w+$'):
+    def envGetList(self, name, attr_regex = '^\w+$', conv=None):
         """Parse the plugin environment variables to return list from variable
         with name list_<name>. The value of the variable must be a comma 
         separated list of items.
@@ -170,6 +228,7 @@ class MuninPlugin:
         @param attr_regex: If the regex is defined, the items in the list are 
                            ignored unless they comply with the format dictated 
                            by the match regex.
+        @param conv:       Function for converting value to desired type.
         @return:           List of items.
         
         """
@@ -178,12 +237,16 @@ class MuninPlugin:
         if self._env.has_key(key):
             if attr_regex:
                 recomp = re.compile(attr_regex)
-                for attr in self._env[key].split(','):
-                    attr = attr.strip()
-                    if recomp.search(attr):
-                        item_list.append(attr) 
             else:
-                item_list = [attr.strip() for attr in self._env[key].split(',')]
+                recomp = None
+            for attr in self._env[key].split(','):
+                attr = attr.strip()
+                if recomp is None or recomp.search(attr):
+                    if conv is not None:
+                        item_list.append(conv(attr))
+                    else:
+                        item_list.append(attr)
+                    
         return item_list
     
     def envRegisterFilter(self, name, attr_regex = '^\w+$', default = True):
@@ -259,14 +322,14 @@ class MuninPlugin:
         """
         return self._debug
     
-    def graphEnabled(self, name):
+    def graphEnabled(self, graph_name):
         """Utility method to check if graph with the given name is enabled.
         
-        @param name: Name of Root Graph Instance.
-        @return:     Returns True if Root Graph is enabled, False otherwise.
+        @param graph_name: Name of Root Graph Instance.
+        @return:           Returns True if graph is enabled, False otherwise.
             
         """
-        return self.envCheckFilter('graphs', name)
+        return self.envCheckFilter('graphs', graph_name)
         
     def saveState(self,  stateObj):
         """Utility methos to save plugin state stored in stateObj to persistent 
@@ -303,18 +366,18 @@ class MuninPlugin:
             return stateObj
         return None
         
-    def appendGraph(self, name, graph):
+    def appendGraph(self, graph_name, graph):
         """Utility method to associate Graph Object to Plugin.
         
         This utility method is for use in constructor of child classes for
         associating a MuninGraph instances to the plugin.
         
-        @param name:  Graph Name
-        @param graph: MuninGraph Instance
+        @param graph_name:  Graph Name
+        @param graph:       MuninGraph Instance
 
         """
-        self._graphDict[name] = graph
-        self._graphNames.append(name)
+        self._graphDict[graph_name] = graph
+        self._graphNames.append(graph_name)
         if not self.isMultigraph  and len(self._graphNames) > 1:
             raise AttributeError("Simple Munin Plugins cannot have more than one graph.")
         
@@ -330,11 +393,13 @@ class MuninPlugin:
 
         """
         if not self.isMultigraph:
-            raise AttributeError("Simple Munin Plugins cannot have more than one graph.")
+            raise AttributeError("Simple Munin Plugins cannot have subgraphs.")
         if self._graphDict.has_key(parent_name):
-            if not self._subGraphDict.has_key(parent_name):
-                self._subGraphDict[parent_name] = {}
-            self._subGraphDict[parent_name][graph_name] = graph
+            if not self._subgraphDict.has_key(parent_name):
+                self._subgraphDict[parent_name] = {}
+                self._subgraphNames[parent_name] = []
+            self._subgraphDict[parent_name][graph_name] = graph
+            self._subgraphNames[parent_name].append(graph_name)
         else:
             raise AttributeError("Invalid parent graph name %s used for subgraph %s."
                 % (parent_name,  graph_name))
@@ -344,48 +409,59 @@ class MuninPlugin:
         
         The private method is for use in retrieveVals() method of child classes.
         
-        @param name:    Graph Name
-        @param valDict: Dictionary of monitored values
+        @param graph_name: Graph Name
+        @param field_name: Field Name.
+        @param val:        Value for field.
 
         """
-        graph = self._graphDict.get(graph_name)
-        if graph is not None:
-            if graph.hasField(field_name):
-                graph.setVal(field_name, val)
-            else:
-                raise AttributeError("Invalid field name %s used for setting "
-                                     "value for graph %s." 
-                                     % (field_name, graph_name))
+        graph = self._getGraph(graph_name, True)
+        if graph.hasField(field_name):
+            graph.setVal(field_name, val)
         else:
-            raise AttributeError("Invalid graph name %s used for setting value." 
-                                 % graph_name)
+            raise AttributeError("Invalid field name %s "
+                                 "for graph %s." 
+                                 % (field_name, graph_name))
     
-    def setSubgraphVal(self,  parent_name,  graph_name,  val):
+    def setSubgraphVal(self,  parent_name,  graph_name, field_name, val):
         """Set Value for Field in Subgraph.
 
         The private method is for use in retrieveVals() method of child
         classes.
         
         @param parent_name: Root Graph Name
-        @param name:        Subgraph Name
-        @param valDict:     Dictionary of monitored values
+        @param graph_name:  Subgraph Name
+        @param field_name:  Field Name.
+        @param val:         Value for field.
 
-        """        
-        graph = self._graphDict.get(parent_name)
-        if graph is not None:
-            graph.setVal("%s.%s" % (parent_name, graph_name),  val)
+        """
+        subgraph = self._getSubGraph(parent_name, graph_name, True)
+        if subgraph.hasField(field_name):
+            subgraph.setVal(field_name, val)
         else:
-            raise AttributeError("Invalid parent graph name %s used "
-                                 "for setting value for subgraph %s."
-                                 % (parent_name, graph_name))
+            raise AttributeError("Invalid field name %s "
+                                 "for subgraph %s "
+                                 "of parent graph %s." 
+                                 % (field_name, graph_name, parent_name))
     
-    def hasGraph(self, name):
+    def hasGraph(self, graph_name):
         """Return true if graph with name is registered to plugin.
         
-        @return: Boolean
+        @param graph_name: Graph Name
+        @return:           Boolean
         
         """
-        return self._graphDict.has_key(name)
+        return self._graphDict.has_key(graph_name)
+    
+    def hasSubgraph(self, parent_name, graph_name):
+        """Return true if Root Graph with name parent_name has a subgraph with 
+        name graph_name.
+        
+        @param parent_name: Root Graph Name
+        @param graph_name:  Subgraph Name
+        @return:            Boolean
+        
+        """
+        return self._getSubGraph(parent_name, graph_name) is not None
             
     def getGraphList(self):
         """Returns list of names of graphs registered to plugin.
@@ -394,23 +470,67 @@ class MuninPlugin:
         
         """
         return self._graphNames
+    
+    def getSubgraphList(self, parent_name):
+        """Returns list of names of subgraphs for Root Graph with name parent_name.
+        
+        @param parent_name: Name of Root Graph.
+        @return -           List of subgraph names.
+        
+        """
+        if not self.isMultigraph:
+            raise AttributeError("Simple Munin Plugins cannot have subgraphs.")
+        if self._graphDict.has_key(parent_name):
+            return self._subgraphNames.get(parent_name) or []
+        else:
+            raise AttributeError("Invalid parent graph name %s."
+                                 % (parent_name,))
 
     def graphHasField(self, graph_name, field_name):
         """Return true if graph with name graph_name has field with 
         name field_name.
         
+        @param graph_name: Graph Name
+        @param field_name: Field Name.
         @return: Boolean
         
         """
-        return self._graphDict[graph_name].hasField(field_name)
+        graph = self._graphDict.get(graph_name, True)
+        return graph.hasField(field_name)
+    
+    def subGraphHasField(self, parent_name, graph_name, field_name):
+        """Return true if subgraph with name graph_name with parent graph with
+        name parent_name has field with name field_name.
+        
+        @param parent_name: Root Graph Name
+        @param graph_name:  Subgraph Name
+        @param field_name:  Field Name.
+        @return:            Boolean
+        
+        """
+        subgraph = self._getSubGraph(parent_name, graph_name, True)
+        return subgraph.hasField(field_name)
             
     def getGraphFieldList(self, graph_name):
         """Returns list of names of fields for graph with name graph_name.
         
-        @return - List of field names for graph.
+        @param graph_name: Graph Name
+        @return:           List of field names for graph.
         
         """
-        return self._graphDict[graph_name].getFieldList()
+        graph = self._getGraph(graph_name, True)
+        return graph.getFieldList()
+    
+    def getSubgraphFieldList(self, parent_name, graph_name):
+        """Returns list of names of fields for graph with name graph_name.
+        
+        @param parent_name: Root Graph Name
+        @param graph_name:  Subgraph Name
+        @return:            List of field names for subgraph.
+        
+        """
+        graph = self._getSubGraph(parent_name, graph_name, True)
+        return graph.getFieldList()
         
     def retrieveVals(self):
         """Initialize measured values for Graphs.
@@ -441,15 +561,16 @@ class MuninPlugin:
         populated.
 
         """
-        for name in self._graphNames:
-            graph = self._graphDict[name]
+        for parent_name in self._graphNames:
+            graph = self._graphDict[parent_name]
             if self.isMultigraph:
-                print "multigraph %s" % name
+                print "multigraph %s" % parent_name
             print graph.getConfig()
             print
-        if self._nestedGraphs and self._subGraphDict:
-            for (parent_name, subgraphs) in self._subGraphDict.iteritems():
-                for (graph_name,  graph) in subgraphs.iteritems():
+        if self._nestedGraphs and self._subgraphDict and self._subgraphNames:
+            for (parent_name, subgraph_names) in self._subgraphNames.iteritems():
+                for graph_name in subgraph_names:
+                    graph = self._subgraphDict[parent_name][graph_name]
                     print "multigraph %s.%s" % (parent_name, graph_name)
                     print graph.getConfig()
                     print
@@ -471,15 +592,16 @@ class MuninPlugin:
 
         """
         self.retrieveVals()
-        for name in self._graphNames:
-            graph = self._graphDict[name]
+        for parent_name in self._graphNames:
+            graph = self._graphDict[parent_name]
             if self.isMultigraph:
-                print "multigraph %s" % name
+                print "multigraph %s" % parent_name
             print graph.getVals()
             print
-        if self._nestedGraphs and self._subGraphDict:
-            for (parent_name, subgraphs) in self._subGraphDict.iteritems():
-                for (graph_name,  graph) in subgraphs.iteritems():
+        if self._nestedGraphs and self._subgraphDict and self._subgraphNames:
+            for (parent_name, subgraph_names) in self._subgraphNames.iteritems():
+                for graph_name in subgraph_names:
+                    graph = self._subgraphDict[parent_name][graph_name]
                     print "multigraph %s.%s" % (parent_name,  graph_name)
                     print graph.getVals()
                     print
