@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 """fsstats - Munin Plugin to monitor FreeSWITCH through the Event Socket 
 Interface.
 
@@ -65,7 +65,7 @@ class MuninFreeswitchPlugin(MuninPlugin):
         MuninPlugin.__init__(self, argv, env, debug)
 
         self._fshost = self.envGet('fshost')
-        self._fsport = self.envGet('fsport')
+        self._fsport = self.envGet('fsport', None, int)
         self._fspass = self.envGet('fspass')
 
         if self.graphEnabled('fs_calls'):
